import { createErrorMap } from '@genesislcap/foundation-utils';
import { assign } from 'xstate';
import { escalate } from 'xstate/actions';
import { logger } from '../utils';
import { isErrorEvent } from './guards';
/**
 * Set error action.
 * @remarks
 * Used to set a top level error on the machine. This may equal to {@link @genesislcap/foundation-utils#ErrorMap} `lastError` in the `errors` property.
 * @public
 */
export const setError = assign(({ event }) => {
    logger.debug('setError', event, isErrorEvent(event));
    if (isErrorEvent(event)) {
        return {
            error: event.data,
        };
    }
});
/**
 * Set error by key action.
 * @remarks
 * Used to set an error in the {@link @genesislcap/foundation-utils#ErrorMap} by key.
 * @param key - The error key.
 * @public
 */
export const setErrorByKey = (key) => assign(({ context, event }) => {
    var _a;
    if (isErrorEvent(event)) {
        const errors = (_a = context.errors) !== null && _a !== void 0 ? _a : createErrorMap(logger.error);
        errors.set(key, event.data);
        return {
            errors,
        };
    }
});
/**
 * Escalate error action.
 * @remarks
 * Guard this call with a parent check to avoid warnings when there's no parent.
 * @public
 */
export const escalateError = escalate(({ context }) => ({
    error: context.error,
    data: context.error,
}));
